# -*- coding: utf-8 -*-
from zgrobot.utils import to_text


def get_value(instance, path, default=None):
    dic = instance.__dict__
    for entry in path.split('.'):
        dic = dic.get(entry)
        if dic is None:
            return default
    return dic or default


class BaseEntry(object):
    def __init__(self, entry, default=None):
        self.entry = entry
        self.default = default


class IntEntry(BaseEntry):
    def __get__(self, instance, owner):
        try:
            return int(get_value(instance, self.entry, self.default))
        except TypeError:
            return


class FloatEntry(BaseEntry):
    def __get__(self, instance, owner):
        try:
            return float(get_value(instance, self.entry, self.default))
        except TypeError:
            return


class StringEntry(BaseEntry):
    def __get__(self, instance, owner):
        v = get_value(instance, self.entry, self.default)
        if v is not None:
            return to_text(v)
        return v



# -*- coding: utf-8 -*-

from zgrobot.utils import to_binary

_BLOCK_SIZE = 32


def encode(text):
    # 计算需要填充的位数
    amount_to_pad = _BLOCK_SIZE - (len(text) % _BLOCK_SIZE)
    if not amount_to_pad:
        amount_to_pad = _BLOCK_SIZE
    # 获得补位所用的字符
    pad = chr(amount_to_pad)
    return text + to_binary(pad * amount_to_pad)



# -*- coding: utf-8 -*-

from zgrobot.messages.entries import get_value, StringEntry, FloatEntry, IntEntry
from zgrobot.utils import to_text


class NoNameMessage(object):
    test_int = IntEntry("TestInt")
    test_string_to_int = IntEntry("TestStringToInt")
    test_float_to_int = IntEntry("TestFloatToInt")
    test_int_none = IntEntry("MIAOMIAOMIAO")

    test_float = FloatEntry("TestFloat")
    test_string_to_float = FloatEntry("TestStringToFloat")
    test_float_none = FloatEntry("WANGWANG")

    test_string = StringEntry("TestString")
    test_int_to_string = StringEntry("TestIntToString")
    test_float_to_string = StringEntry("TestFloatToString")
    test_chinese = StringEntry("TestChinese")
    test_string_none = StringEntry("HAHAHA")

    def __init__(self):
        message = {
            "TestInt": 123,
            "TestFloat": 0.00001,
            "TestString": "hello",
            "TestStringToInt": "123",
            "TestFloatToInt": 123.000,
            "TestStringToFloat": "0.00001",
            "TestIntToString": 123,
            "TestFloatToString": 0.00001,
            "TestChinese": "喵",
        }
        self.__dict__.update(message)


t = NoNameMessage()


def test_int_entry():
    assert isinstance(t.test_int, int)
    assert t.test_int == 123
    assert isinstance(t.test_string_to_int, int)
    assert t.test_string_to_int == 123
    assert isinstance(t.test_float_to_int, int)
    assert t.test_float_to_int == 123
    assert t.test_int_none is None


def test_float_entry():
    assert isinstance(t.test_float, float)
    assert t.test_float == 0.00001
    assert isinstance(t.test_string_to_float, float)
    assert t.test_string_to_float == 0.00001
    assert t.test_float_none is None


def test_string_entry():
    assert isinstance(t.test_string, str)
    assert t.test_string == "hello"
    assert isinstance(t.test_int_to_string, str)
    assert t.test_int_to_string == "123"
    assert isinstance(t.test_float_to_string, str)
    assert t.test_float_to_string == "1e-05"
    assert isinstance(t.test_chinese, str)
    assert t.test_chinese == to_text("喵")
    assert t.test_string_none is None


class FakeIntance:
    pass


def test_get_value():
    instance = FakeIntance()
    instance.b = 6
    instance.a = {'c': 'd'}
    assert get_value(instance, 'd', 'default') == 'default'
    assert get_value(instance, 'b', 'default') == 6
    assert get_value(instance, 'a.c') == 'd'



# -*- coding: utf-8 -*-

from zgrobot import ZgRoBot
from zgrobot.parser import parse_user_msg
from zgrobot.replies import TextReply
import os

zgrobot = ZgRoBot(SESSION_STORAGE=False)


def teardown_module(module):
    try:
        os.remove(os.path.abspath("zgrobot_session"))
    except OSError:
        pass


def test_subscribe_handler():
    @zgrobot.subscribe
    def subscribe(message):
        return '关注'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[FromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[subscribe]]></Event>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'关注'


def test_unsubscribe_handler():
    @zgrobot.unsubscribe
    def unsubscribe(message):
        return '取消关注'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[unsubscribe]]></Event>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'取消关注'


def test_scan_push_handler():
    @zgrobot.scancode_push
    def scancode_push(message):
        return '扫描推送'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[FromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[scancode_push]]></Event>
            <EventKey><![CDATA[EVENTKEY]]></EventKey>
            <ScanCodeInfo>
                <ScanType><![CDATA[qrcode]]></ScanType>
                <ScanResult><![CDATA[http://www.qq.com]]></ScanResult>
            </ScanCodeInfo>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'扫描推送'


def test_scan_waitmsg_handler():
    @zgrobot.scancode_waitmsg
    def scancode_waitmsg(message):
        return '扫描弹消息'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[FromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[scancode_waitmsg]]></Event>
            <EventKey><![CDATA[EVENTKEY]]></EventKey>
            <ScanCodeInfo>
                <ScanType><![CDATA[qrcode]]></ScanType>
                <ScanResult><![CDATA[http://www.qq.com]]></ScanResult>
            </ScanCodeInfo>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'扫描弹消息'


def test_pic_sysphoto_handler():
    @zgrobot.pic_sysphoto
    def pic_sysphoto():
        return '瞧一瞧系统拍照'

    message = parse_user_msg(
        """
    <xml>
        <ToUserName><![CDATA[gh_e136c6e50636]]></ToUserName>
        <FromUserName><![CDATA[oMgHVjngRipVsoxg6TuX3vz6glDg]]></FromUserName>
        <CreateTime>1408090651</CreateTime>
        <MsgType><![CDATA[event]]></MsgType>
        <Event><![CDATA[pic_sysphoto]]></Event>
        <EventKey><![CDATA[6]]></EventKey>
        <SendPicsInfo>
            <Count>1</Count>
            <PicList>
                <item>
                    <PicMd5Sum><![CDATA[1b5f7c23b5bf75682a53e7b6d163e185]]></PicMd5Sum>
                </item>
            </PicList>
        </SendPicsInfo>
    </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'瞧一瞧系统拍照'


def test_pic_photo_or_album_handler():
    @zgrobot.pic_photo_or_album
    def pic_photo_or_album():
        return '瞧一瞧拍照或者相册'

    message = parse_user_msg(
        """
    <xml>
        <ToUserName><![CDATA[gh_e136c6e50636]]></ToUserName>
        <FromUserName><![CDATA[oMgHVjngRipVsoxg6TuX3vz6glDg]]></FromUserName>
        <CreateTime>1408090816</CreateTime>
        <MsgType><![CDATA[event]]></MsgType>
        <Event><![CDATA[pic_photo_or_album]]></Event>
        <EventKey><![CDATA[6]]></EventKey>
        <SendPicsInfo>
            <Count>1</Count>
            <PicList>
                <item>
                    <PicMd5Sum><![CDATA[5a75aaca956d97be686719218f275c6b]]></PicMd5Sum>
                </item>
            </PicList>
        </SendPicsInfo>
    </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'瞧一瞧拍照或者相册'


def test_pic_weixin_handler():
    @zgrobot.pic_weixin
    def pic_weixin():
        return '瞧一瞧微信相册'

    message = parse_user_msg(
        """
    <xml>
        <ToUserName><![CDATA[gh_e136c6e50636]]></ToUserName>
        <FromUserName><![CDATA[oMgHVjngRipVsoxg6TuX3vz6glDg]]></FromUserName>
        <CreateTime>1408090816</CreateTime>
        <MsgType><![CDATA[event]]></MsgType>
        <Event><![CDATA[pic_weixin]]></Event>
        <EventKey><![CDATA[6]]></EventKey>
        <SendPicsInfo>
            <Count>1</Count>
            <PicList>
                <item>
                    <PicMd5Sum><![CDATA[5a75aaca956d97be686719218f275c6b]]></PicMd5Sum>
                </item>
            </PicList>
        </SendPicsInfo>
    </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'瞧一瞧微信相册'


def test_location_select_handler():
    @zgrobot.location_select
    def location_select():
        return '瞧一瞧地理位置'

    message = parse_user_msg(
        """
    <xml>
        <ToUserName><![CDATA[gh_e136c6e50636]]></ToUserName>
        <FromUserName><![CDATA[oMgHVjngRipVsoxg6TuX3vz6glDg]]></FromUserName>
        <CreateTime>1408091189</CreateTime>
        <MsgType><![CDATA[event]]></MsgType>
        <Event><![CDATA[location_select]]></Event>
        <EventKey><![CDATA[6]]></EventKey>
        <SendLocationInfo>
            <Location_X><![CDATA[23]]></Location_X>
            <Location_Y><![CDATA[113]]></Location_Y>
            <Scale><![CDATA[15]]></Scale>
            <Label><![CDATA[广州市海珠区客村艺苑路 106号]]></Label>
            <Poiname><![CDATA[]]></Poiname>
        </SendLocationInfo>
    </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'瞧一瞧地理位置'


def test_click_handler():
    @zgrobot.click
    def scan(message):
        return '喵喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[CLICK]]></Event>
            <EventKey><![CDATA[EVENTKEY]]></EventKey>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'喵喵'


def test_view_handler():
    @zgrobot.view
    def view(message):
        return '汪汪'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[FromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[VIEW]]></Event>
            <EventKey><![CDATA[www.qq.com]]></EventKey>
        </xml>"""
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'汪汪'


def test_location_event_handler():
    @zgrobot.location_event
    def location_event(message):
        return '位置喵喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[LOCATION]]></Event>
            <Latitude>23.137466</Latitude>
            <Longitude>113.352425</Longitude>
            <Precision>119.385040</Precision>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'位置喵喵'


def test_card_pass_check_handler():
    @zgrobot.card_pass_check
    def card_pass_check():
        return '瞧一瞧通过了'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[FromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[card_pass_check]]></Event>
            <CardId><![CDATA[cardid]]></CardId>
            <RefuseReason><![CDATA[非法代制]]></RefuseReason>
        </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'瞧一瞧通过了'


def test_card_not_pass_check_handler():
    @zgrobot.card_not_pass_check
    def card_pass_not_check():
        return '瞧一瞧没过'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[FromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[card_not_pass_check]]></Event>
            <CardId><![CDATA[cardid]]></CardId>
            <RefuseReason><![CDATA[非法代制]]></RefuseReason>
        </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'瞧一瞧没过'


def test_user_get_card_handler():
    @zgrobot.user_get_card
    def user_get_card():
        return '恭喜入坑'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName> <![CDATA[gh_fc0a06a20993]]> </ToUserName>
                <FromUserName> <![CDATA[oZI8Fj040-be6rlDohc6gkoPOQTQ]]> </FromUserName>
                <CreateTime>1472551036</CreateTime>
                <MsgType> <![CDATA[event]]> </MsgType>
                <Event> <![CDATA[user_get_card]]> </Event>
                <CardId> <![CDATA[pZI8Fjwsy5fVPRBeD78J4RmqVvBc]]> </CardId>
                <IsGiveByFriend>0</IsGiveByFriend>
                <UserCardCode> <![CDATA[226009850808]]> </UserCardCode>
                <FriendUserName> <![CDATA[]]> </FriendUserName>
                <OuterId>0</OuterId>
                <OldUserCardCode> <![CDATA[]]> </OldUserCardCode>
                <OuterStr> <![CDATA[12b]]> </OuterStr>
                <IsRestoreMemberCard>0</IsRestoreMemberCard>
                <IsRecommendByFriend>0</IsRecommendByFriend>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'恭喜入坑'


def test_user_gifting_card_handler():
    @zgrobot.user_gifting_card
    def user_gifting_card():
        return '锅从天上来'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName><![CDATA[gh_3fcea188bf78]]></ToUserName>
                <FromUserName><![CDATA[obLatjjwDolFjRRd3doGIdwNqRXw]]></FromUserName>
                <CreateTime>1474181868</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[user_gifting_card]]></Event>
                <CardId><![CDATA[pbLatjhU-3pik3d4PsbVzvBxZvJc]]></CardId>
                <UserCardCode><![CDATA[297466945104]]></UserCardCode>
                <IsReturnBack>0</IsReturnBack>
                <FriendUserName><![CDATA[obLatjlNerkb62HtSdQUx66C4NTU]]></FriendUserName>
                <IsChatRoom>0</IsChatRoom>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'锅从天上来'


def test_user_del_card_handler():
    @zgrobot.user_del_card
    def user_del_card():
        return '摆脱负担'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName><![CDATA[toUser]]></ToUserName>
                <FromUserName><![CDATA[FromUser]]></FromUserName>
                <CreateTime>123456789</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[user_del_card]]></Event>
                <CardId><![CDATA[cardid]]></CardId>
                <UserCardCode><![CDATA[12312312]]></UserCardCode>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'摆脱负担'


def test_user_consume_card_handler():
    @zgrobot.user_consume_card
    def user_consume_card():
        return '恭喜脱坑'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName> <![CDATA[gh_fc0a06a20993]]> </ToUserName>
                <FromUserName> <![CDATA[oZI8Fj040-be6rlDohc6gkoPOQTQ]]> </FromUserName>
                <CreateTime>1472549042</CreateTime>
                <MsgType> <![CDATA[event]]> </MsgType>
                <Event> <![CDATA[user_consume_card]]> </Event>
                <CardId> <![CDATA[pZI8Fj8y-E8hpvho2d1ZvpGwQBvA]]> </CardId>
                <UserCardCode> <![CDATA[452998530302]]> </UserCardCode>
                <ConsumeSource> <![CDATA[FROM_API]]> </ConsumeSource>
                <LocationName> <![CDATA[]]> </LocationName>
                <StaffOpenId> <![CDATA[oZ********nJ3bPJu_Rtjkw4c]]> </StaffOpenId>
                <VerifyCode> <![CDATA[]]> </VerifyCode>
                <RemarkAmount> <![CDATA[]]> </RemarkAmount>
                <OuterStr> <![CDATA[xxxxx]]> </OuterStr>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'恭喜脱坑'


def test_user_pay_from_pay_cell_handler():
    @zgrobot.user_pay_from_pay_cell
    def user_pay_from_pay_cell():
        return '冲动消费'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName><![CDATA[gh_e2243xxxxxxx]]></ToUserName>
                <FromUserName><![CDATA[oo2VNuOUuZGMxxxxxxxx]]></FromUserName>
                <CreateTime>1442390947</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[user_pay_from_pay_cell]]></Event>
                <CardId><![CDATA[po2VNuCuRo-8sxxxxxxxxxxx]]></CardId>
                <UserCardCode><![CDATA[38050000000]]></UserCardCode>
                <TransId><![CDATA[10022403432015000000000]]></TransId>
                <LocationId>291710000</LocationId>
                <Fee><![CDATA[10000]]></Fee>
                <OriginalFee><![CDATA[10000]]> </OriginalFee>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'冲动消费'


def test_user_view_card_handler():
    @zgrobot.user_view_card
    def user_view_card():
        return '我就瞧一瞧，不买'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName> <![CDATA[gh_fcxxxx6a20993]]> </ToUserName>
                <FromUserName> <![CDATA[oZI8Fj040-xxxxx6gkoPOQTQ]]> </FromUserName>
                <CreateTime>1467811138</CreateTime>
                <MsgType> <![CDATA[event]]> </MsgType>
                <Event> <![CDATA[user_view_card]]> </Event>
                <CardId> <![CDATA[pZI8Fj2ezBbxxxxxT2UbiiWLb7Bg]]> </CardId>
                <UserCardCode> <![CDATA[4xxxxxxxx8558]]> </UserCardCode>
                <OuterStr> <![CDATA[12b]]> </OuterStr>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'我就瞧一瞧，不买'


def test_user_enter_session_from_card_handler():
    @zgrobot.user_enter_session_from_card
    def user_enter_session_from_card():
        return '退货是不可能退货的'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName><![CDATA[toUser]]></ToUserName>
                <FromUserName><![CDATA[FromUser]]></FromUserName>
                <CreateTime>123456789</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[user_enter_session_from_card]]></Event>
                <CardId><![CDATA[cardid]]></CardId>
                <UserCardCode><![CDATA[12312312]]></UserCardCode>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'退货是不可能退货的'


def test_update_member_card_handler():
    @zgrobot.update_member_card
    def update_member_card():
        return '冲动消费导致余额减少'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName><![CDATA[gh_9e1765b5568e]]></ToUserName>
                <FromUserName><![CDATA[ojZ8YtyVyr30HheH3CM73y7h4jJE]]></FromUserName>
                <CreateTime>1445507140</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[update_member_card]]></Event>
                <CardId><![CDATA[pjZ8Ytx-nwvpCRyQneH3Ncmh6N94]]></CardId>
                <UserCardCode><![CDATA[485027611252]]></UserCardCode>
                <ModifyBonus>3</ModifyBonus>
                <ModifyBalance>0</ModifyBalance>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'冲动消费导致余额减少'


def test_card_sku_remind_handler():
    @zgrobot.card_sku_remind
    def card_sku_remind():
        return '骗钱大成功'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName><![CDATA[gh_2d62d*****0]]></ToUserName>
                <FromUserName><![CDATA[oa3LFuBvWb7*********]]></FromUserName>
                <CreateTime>1443838506</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[card_sku_remind]]></Event>
                <CardId><![CDATA[pa3LFuAh2P65**********]]></CardId>
                <Detail><![CDATA[the card's quantity is equal to 0]]></Detail>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'骗钱大成功'


def test_card_pay_order_handler():
    @zgrobot.card_pay_order
    def card_pay_order():
        return '冲动消费的凭证'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName><![CDATA[gh_7223c83d4be5]]></ToUserName>
                <FromUserName><![CDATA[ob5E7s-HoN9tslQY3-0I4qmgluHk]]></FromUserName>
                <CreateTime>1453295737</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[card_pay_order]]></Event>
                <OrderId><![CDATA[404091456]]></OrderId>
                <Status><![CDATA[ORDER_STATUS_FINANCE_SUCC]]></Status>
                <CreateOrderTime>1453295737</CreateOrderTime>
                <PayFinishTime>0</PayFinishTime>
                <Desc><![CDATA[]]></Desc>
                <FreeCoinCount><![CDATA[200]]></FreeCoinCount>
                <PayCoinCount><![CDATA[0]]></PayCoinCount>
                <RefundFreeCoinCount><![CDATA[0]]></RefundFreeCoinCount>
                <RefundPayCoinCount><![CDATA[0]]></RefundPayCoinCount>
                <OrderType><![CDATA[ORDER_TYPE_SYS_ADD]]></OrderType>
                <Memo><![CDATA[开通账户奖励]]></Memo>
                <ReceiptInfo><![CDATA[]]></ReceiptInfo>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'冲动消费的凭证'


def test_submit_membercard_user_info_handler():
    @zgrobot.submit_membercard_user_info
    def submit_membercard_user_info():
        return '现在醒一醒还来得及'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName> <![CDATA[gh_3fcea188bf78]]></ToUserName>
                <FromUserName><![CDATA[obLatjlaNQKb8FqOvt1M1x1lIBFE]]></FromUserName>
                <CreateTime>1432668700</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[submit_membercard_user_info]]></Event>
                <CardId><![CDATA[pbLatjtZ7v1BG_ZnTjbW85GYc_E8]]></CardId>
                <UserCardCode><![CDATA[018255396048]]></UserCardCode>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'现在醒一醒还来得及'


def test_templatesendjobfinish_event_handler():
    @zgrobot.templatesendjobfinish_event
    def templatesendjobfinish_event():
        return '喵喵~模板消息已经推送'

    message = parse_user_msg(
        """
            <xml>
                <ToUserName><![CDATA[gh_7f083739789a]]></ToUserName>
                <FromUserName><![CDATA[oia2TjuEGTNoeX76QEjQNrcURxG8]]></FromUserName>
                <CreateTime>1395658920</CreateTime>
                <MsgType><![CDATA[event]]></MsgType>
                <Event><![CDATA[TEMPLATESENDJOBFINISH]]></Event>
                <MsgID>200163836</MsgID>
                <Status><![CDATA[success]]></Status>
            </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'喵喵~模板消息已经推送'


def test_unknown_event():
    @zgrobot.unknown_event
    def unknown_event(message):
        return '不知道的事件喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[FromUser]]></FromUserName>
            <CreateTime>123456789</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[unknown]]></Event>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'不知道的事件喵'


def test_text():
    @zgrobot.text
    def text(message):
        return '普通的Text喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>1348831860</CreateTime>
            <MsgType><![CDATA[text]]></MsgType>
            <Content><![CDATA[this is a test]]></Content>
            <MsgId>1234567890123456</MsgId>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'普通的Text喵'


def test_image():
    @zgrobot.image
    def image(message):
        return '图片喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>1348831860</CreateTime>
            <MsgType><![CDATA[image]]></MsgType>
            <PicUrl><![CDATA[this is a url]]></PicUrl>
            <MsgId>1234567890123456</MsgId>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'图片喵'


def test_location():
    @zgrobot.location
    def location(message):
        return '地理位置汪'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>1351776360</CreateTime>
            <MsgType><![CDATA[location]]></MsgType>
            <Location_X>23.134521</Location_X>
            <Location_Y>113.358803</Location_Y>
            <Scale>20</Scale>
            <Label><![CDATA[Location]]></Label>
            <MsgId>1234567890123456</MsgId>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'地理位置汪'


def test_link():
    @zgrobot.link
    def link(message):
        return '链接喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>1351776360</CreateTime>
            <MsgType><![CDATA[link]]></MsgType>
            <Title><![CDATA[ZgRoBot]]></Title>
            <Description><![CDATA[Link to ZgRoBot]]></Description>
            <Url><![CDATA[https://github.com/whtsky/ZgRoBot]]></Url>
            <MsgId>1234567890123456</MsgId>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'链接喵'


def test_voice():
    @zgrobot.voice
    def voice(message):
        return '声音喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>1357290913</CreateTime>
            <MsgType><![CDATA[voice]]></MsgType>
            <MediaId><![CDATA[media_id]]></MediaId>
            <Format><![CDATA[Format]]></Format>
            <Recognition><![CDATA[Meow~]]></Recognition>
            <MsgId>1234567890123456</MsgId>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'声音喵'


def test_video():
    @zgrobot.video
    def video():
        return '请收下这一段榴莲的视频'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>1357290913</CreateTime>
            <MsgType><![CDATA[video]]></MsgType>
            <MediaId><![CDATA[media_id]]></MediaId>
            <ThumbMediaId><![CDATA[thumb_media_id]]></ThumbMediaId>
            <MsgId>1234567890123456</MsgId>
            </xml>"""
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'请收下这一段榴莲的视频'


def test_shortvideo():
    @zgrobot.shortvideo
    def shortvideo():
        return '请收下这一段榴莲的小视频'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>1357290913</CreateTime>
            <MsgType><![CDATA[shortvideo]]></MsgType>
            <MediaId><![CDATA[media_id]]></MediaId>
            <ThumbMediaId><![CDATA[thumb_media_id]]></ThumbMediaId>
            <MsgId>1234567890123456</MsgId>
        </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'请收下这一段榴莲的小视频'


def test_user_scan_product():
    @zgrobot.user_scan_product
    def user_scan_product():
        return '打扰了'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[gh_4e47c9c9ecad]]></ToUserName>
            <FromUserName><![CDATA[okkeXs1nI-xU4ql8-5BXkv1f0gDo]]></FromUserName>
            <CreateTime>1438250110</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[user_scan_product]]></Event>
            <KeyStandard><![CDATA[ean13]]></KeyStandard>
            <KeyStr><![CDATA[6901481811083]]></KeyStr>
            <Country><![CDATA[中国]]></Country>
            <Province><![CDATA[广东]]></Province>
            <City><![CDATA[揭阳]]></City>
            <Sex>1</Sex>
            <Scene>2</Scene>
            <ExtInfo><![CDATA[123]]></ExtInfo>
        </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'打扰了'


def test_user_scan_product_enter_session():
    @zgrobot.user_scan_product_enter_session
    def user_scan_product_enter_session():
        return '再次打扰了'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[gh_fbe8a958756e]]></ToUserName>
            <FromUserName><![CDATA[otAzGjrS4AYCmeJM1GhEOcHXXTAo]]></FromUserName>
            <CreateTime>1433259128</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[user_scan_product_enter_session]]></Event>
            <KeyStandard><![CDATA[ena13]]></KeyStandard>
            <KeyStr><![CDATA[6954767461373]]></KeyStr>
            <ExtInfo><![CDATA[]]></ExtInfo>
        </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'再次打扰了'


def test_user_scan_product_async():
    @zgrobot.user_scan_product_async
    def user_scan_product_async():
        return '异步的地理位置喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[gh_fbe8a958756e]]></ToUserName>
            <FromUserName><![CDATA[otAzGjrS4AYCmeJM1GhEOcHXXTAo]]></FromUserName>
            <CreateTime>1434541327</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[user_scan_product_async]]></Event>
            <KeyStandard><![CDATA[qrcode]]></KeyStandard>
            <KeyStr><![CDATA[lincolntest2]]></KeyStr>
            <ExtInfo><![CDATA[123]]></ExtInfo>
            <RegionCode><![CDATA[440105]]></RegionCode>
        </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'异步的地理位置喵'


def test_user_scan_product_verify_action():
    @zgrobot.user_scan_product_verify_action
    def user_scan_product_verify_action():
        return '审核通过了喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[gh_404e58ec528e]]></ToUserName>
            <FromUserName><![CDATA[od_ikt8qi21-hVTtYgm8xSfTLH5w]]></FromUserName>
            <CreateTime>1450429257</CreateTime>
            <MsgType><![CDATA[event]]></MsgType>
            <Event><![CDATA[user_scan_product_verify_action]]></Event>
            <KeyStandard><![CDATA[ean13]]></KeyStandard>
            <KeyStr><![CDATA[6901481811083]]></KeyStr>
            <Result><![CDATA[verify_ok]]></Result>
            <ReasonMsg><![CDATA[]]></ReasonMsg>
        </xml>
    """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'审核通过了喵'


def test_unknown():
    @zgrobot.unknown
    def unknown(message):
        return '不知道喵'

    message = parse_user_msg(
        """
        <xml>
            <ToUserName><![CDATA[toUser]]></ToUserName>
            <FromUserName><![CDATA[fromUser]]></FromUserName>
            <CreateTime>1351776360</CreateTime>
            <MsgType><![CDATA[unknown]]></MsgType>
            <Title><![CDATA[ZgRoBot]]></Title>
            <Description><![CDATA[Link to ZgRoBot]]></Description>
            <Url><![CDATA[https://github.com/whtsky/ZgRoBot]]></Url>
            <MsgId>1234567890123456</MsgId>
        </xml>
        """
    )

    reply = zgrobot.get_reply(message)

    assert isinstance(reply, TextReply)
    assert reply._args['content'] == u'不知道喵'



# -*- coding: utf-8 -*-

try:
    import anydbm as dbm

    assert dbm
except ImportError:
    import dbm

from zgrobot.session import SessionStorage
from zgrobot.utils import json_loads, json_dumps, to_binary


class FileStorage(SessionStorage):
    """
    FileStorage 会把你的 Session 数据以 dbm 形式储存在文件中。

    :param filename: 文件名， 默认为 ``zgrobot_session``
    """
    def __init__(self, filename: str = 'zgrobot_session'):
        try:
            self.db = dbm.open(filename, "c")
        except TypeError:  # pragma: no cover
            # dbm in PyPy requires filename to be binary
            self.db = dbm.open(to_binary(filename), "c")

    def get(self, id):
        """
        根据 id 获取数据。

        :param id: 要获取的数据的 id
        :return: 返回取到的数据，如果是空则返回一个空的 ``dict`` 对象
        """
        try:
            session_json = self.db[id]
        except KeyError:
            session_json = "{}"
        return json_loads(session_json)

    def set(self, id, value):
        """
        根据 id 写入数据。

        :param id: 要写入的 id
        :param value: 要写入的数据，可以是一个 ``dict`` 对象
        """
        self.db[id] = json_dumps(value)

    def delete(self, id):
        """
        根据 id 删除数据。

        :param id: 要删除的数据的 id
        """
        del self.db[id]



# -*- coding: utf-8 -*-

import os
import mongomock
import mockredis
import pytest
import sqlite3

import zgrobot
import zgrobot.testing
import zgrobot.utils
from zgrobot.session import SessionStorage
from zgrobot.session import filestorage, mongodbstorage, redisstorage, saekvstorage
from zgrobot.session import sqlitestorage
from zgrobot.session import mysqlstorage
from zgrobot.session import postgresqlstorage
from zgrobot.utils import to_binary


def teardown_module():
    try:
        os.remove("zgrobot_session")
        os.remove("zgrobot_session.sqlite3")
    except:
        pass


def remove_session(session):
    try:
        del session[to_binary("fromUser")]
    except:
        pass


def test_session():
    robot = zgrobot.ZgRoBot(
        token=zgrobot.utils.generate_token(), enable_session=True
    )

    @robot.text
    def first(message, session):
        if 'last' in session:
            return
        session['last'] = message.content
        return message.content

    @robot.text
    def second(_, session):
        return session['last']

    tester = zgrobot.testing.WeTest(robot)
    xml_1 = """
        <xml>
        <ToUserName><![CDATA[toUser]]></ToUserName>
        <FromUserName><![CDATA[fromUser]]></FromUserName>
        <CreateTime>1348831860</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[ss]]></Content>
        <MsgId>1234567890123456</MsgId>
        </xml>
    """
    xml_2 = """
        <xml>
        <ToUserName><![CDATA[toUser]]></ToUserName>
        <FromUserName><![CDATA[fromUser]]></FromUserName>
        <CreateTime>1348831860</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[dd]]></Content>
        <MsgId>1234567890123456</MsgId>
        </xml>
    """

    reply_1 = tester.send_xml(xml_1)._args['content']
    assert reply_1 == 'ss'
    reply_2 = tester.send_xml(xml_2)._args['content']
    assert reply_2 == 'ss'


def test_session_storage_get():
    session = SessionStorage()
    with pytest.raises(NotImplementedError):
        session.get('s')
    with pytest.raises(NotImplementedError):
        session['s']


def test_session_storage_set():
    session = SessionStorage()
    with pytest.raises(NotImplementedError):
        session.set('s', {})
    with pytest.raises(NotImplementedError):
        session['s'] = {}


def test_session_storage_delete():
    session = SessionStorage()
    with pytest.raises(NotImplementedError):
        session.delete('s')

    with pytest.raises(NotImplementedError):
        del session['s']


class MockPyMySQL:
    def __init__(self):
        self.db = sqlite3.connect("zgrobot_session.sqlite3")
        self.db.text_factory = str
        from zgrobot.session.sqlitestorage import __CREATE_TABLE_SQL__
        self.db.execute(__CREATE_TABLE_SQL__)
        self.cache_result = None

    def cursor(self):
        return self

    def execute(self, *args, **kwargs):
        if "CREATE TABLE" not in args[0]:
            args = list(args)
            args[0] = args[0].replace('%s', '?')
            if "SELECT" in args[0]:
                self.cache_result = self.db.execute(*args, **kwargs).fetchone()
            elif "INSERT" in args[0]:
                args = [
                    "INSERT OR REPLACE INTO ZgRoBot (id, value) VALUES (?,?);",
                    (args[1][0], args[1][1])
                ]
                self.db.execute(*args, **kwargs)
            else:
                self.db.execute(*args, **kwargs)

    def fetchone(self):
        return self.cache_result

    def commit(self):
        return self.db.commit()


class MockPostgreSQL(MockPyMySQL):
    pass


@pytest.mark.parametrize(
    "storage", [
        filestorage.FileStorage(),
        mongodbstorage.MongoDBStorage(mongomock.MongoClient().t.t),
        redisstorage.RedisStorage(mockredis.mock_redis_client()),
        sqlitestorage.SQLiteStorage(),
        mysqlstorage.MySQLStorage(MockPyMySQL()),
        postgresqlstorage.PostgreSQLStorage(MockPostgreSQL())
    ]
)
def test_storage(storage):
    assert storage.get("喵") == {}
    storage.set("喵", "喵喵")
    assert storage.get("喵") == u"喵喵"
    storage.delete("喵")
    assert storage.get("喵") == {}

    assert storage["榴莲"] == {}
    storage["榴莲"] = "榴莲"
    assert storage["榴莲"] == u"榴莲"
    del storage["榴莲"]
    assert storage["榴莲"] == {}



import os

from zgrobot import ZgRoBot
from zgrobot.config import Config
from zgrobot.utils import generate_token

basedir = os.path.dirname(os.path.abspath(__file__))

TOKEN = "123"


def test_from_pyfile():
    config = Config()
    assert "TOKEN" not in config
    config.from_pyfile(os.path.join(basedir, "test_config.py"))
    assert config["TOKEN"] == "123"


def test_from_object():
    config = Config()
    config.from_pyfile(os.path.join(basedir, "test_config.py"))

    class ConfigObject:
        TOKEN = "456"

    config.from_object(ConfigObject())
    assert config["TOKEN"] == "456"


def test_config_attribute():
    robot = ZgRoBot(SESSION_STORAGE=False)
    assert not robot.token
    token = generate_token()
    robot.config["TOKEN"] = token
    assert robot.token == token

    token = generate_token()
    robot.token = token
    assert robot.config["TOKEN"] == token



# -*- coding: utf-8 -*-

from zgrobot.session import SessionStorage
from zgrobot.utils import json_loads, json_dumps
import sqlite3

__CREATE_TABLE_SQL__ = """
CREATE TABLE IF NOT EXISTS ZgRoBot
(id TEXT PRIMARY KEY NOT NULL ,
value TEXT NOT NULL );
"""


class SQLiteStorage(SessionStorage):
    """
    SQLiteStorge 会把 Session 数据储存在一个 SQLite 数据库文件中 ::

        import zgrobot
        from zgrobot.session.sqlitestorage import SQLiteStorage

        session_storage = SQLiteStorage
        robot = zgrobot.ZgRoBot(token="token", enable_session=True,
                                session_storage=session_storage)

    :param filename: SQLite数据库的文件名, 默认是 ``zgrobot_session.sqlite3``
    """
    def __init__(self, filename='zgrobot_session.sqlite3'):
        self.db = sqlite3.connect(filename, check_same_thread=False)
        self.db.text_factory = str
        self.db.execute(__CREATE_TABLE_SQL__)

    def get(self, id):
        """
        根据 id 获取数据。

        :param id: 要获取的数据的 id
        :return: 返回取到的数据，如果是空则返回一个空的 ``dict`` 对象
        """
        session_json = self.db.execute(
            "SELECT value FROM ZgRoBot WHERE id=? LIMIT 1;", (id, )
        ).fetchone()
        if session_json is None:
            return {}
        return json_loads(session_json[0])

    def set(self, id, value):
        """
        根据 id 写入数据。

        :param id: 要写入的 id
        :param value: 要写入的数据，可以是一个 ``dict`` 对象
        """
        self.db.execute(
            "INSERT OR REPLACE INTO ZgRoBot (id, value) VALUES (?,?);",
            (id, json_dumps(value))
        )
        self.db.commit()

    def delete(self, id):
        """
        根据 id 删除数据。

        :param id: 要删除的数据的 id
        """
        self.db.execute("DELETE FROM ZgRoBot WHERE id=?;", (id, ))
        self.db.commit()



# -*- coding: utf-8 -*-

import re

from zgrobot.utils import generate_token, check_token, to_text, to_binary
from zgrobot.utils import pay_sign_dict, make_error_page, is_regex


def test_token_generator():
    assert not check_token('AA C')
    assert check_token(generate_token())
    assert 3 <= len(generate_token()) <= 32


def test_to_text():
    assert to_text(6) == str(6)
    assert to_text(b"aa") == "aa"
    assert to_text("cc") == "cc"
    assert to_text(u"喵") == u"喵"
    assert to_text("喵") == u"喵"


def test_to_binary():
    assert to_binary(6) == bytes(6)
    assert to_binary(b"aa") == b"aa"
    assert to_binary("cc") == b"cc"
    assert to_binary(u"喵") == b"\xe5\x96\xb5"
    assert to_binary("喵") == b"\xe5\x96\xb5"


def test_pay_sign_dict():
    appid = {"id": "nothing"}
    key = "test_key"

    pay_sign = pay_sign_dict(appid, key)

    assert "timestamp" in pay_sign[0]
    assert "noncestr" in pay_sign[0]
    assert "appid" in pay_sign[0]
    assert pay_sign[0]["appid"] == appid
    assert pay_sign[2] == u"SHA1"

    pay_sign = pay_sign_dict(
        appid, key, add_noncestr=False, add_timestamp=False, gadd_appid=False
    )

    assert "timestamp" not in pay_sign[0]
    assert "noncestr" not in pay_sign[0]
    assert "appid" in pay_sign[0]


def test_make_error_page():
    rand_string = generate_token()
    content = make_error_page(rand_string)
    assert rand_string in content


def test_is_regex():
    regex = re.compile(r"test")
    assert not is_regex("test")
    assert is_regex(regex)



from .parser import parse_user_msg

__all__ = ['WeTest']


class WeTest(object):
    def __init__(self, app):
        self._app = app

    def send_xml(self, xml):
        message = parse_user_msg(xml)
        return self._app.get_reply(message)



# -*- coding: utf-8 -*-

from zgrobot.messages.entries import StringEntry, IntEntry, FloatEntry
from zgrobot.messages.base import ZgRoBotMetaClass


class EventMetaClass(ZgRoBotMetaClass):
    pass


class WeChatEvent(object, metaclass=EventMetaClass):
    target = StringEntry('ToUserName')
    source = StringEntry('FromUserName')
    time = IntEntry('CreateTime')
    message_id = IntEntry('MsgID', 0)

    def __init__(self, message):
        self.__dict__.update(message)


class SimpleEvent(WeChatEvent):
    key = StringEntry('EventKey')


class TicketEvent(WeChatEvent):
    key = StringEntry('EventKey')
    ticket = StringEntry('Ticket')


class SubscribeEvent(TicketEvent):
    __type__ = 'subscribe_event'


class UnSubscribeEvent(WeChatEvent):
    __type__ = 'unsubscribe_event'


class ScanEvent(TicketEvent):
    __type__ = 'scan_event'


class ScanCodePushEvent(SimpleEvent):
    __type__ = 'scancode_push_event'
    scan_type = StringEntry('ScanCodeInfo.ScanType')
    scan_result = StringEntry('ScanCodeInfo.ScanResult')


class ScanCodeWaitMsgEvent(ScanCodePushEvent):
    __type__ = 'scancode_waitmsg_event'
    scan_type = StringEntry('ScanCodeInfo.ScanType')
    scan_result = StringEntry('ScanCodeInfo.ScanResult')


class BasePicEvent(SimpleEvent):
    count = IntEntry('SendPicsInfo.Count')

    def __init__(self, message):
        super(BasePicEvent, self).__init__(message)
        self.pic_list = list()
        if self.count > 1:
            for item in message['SendPicsInfo']['PicList'].pop('item'):
                self.pic_list.append({'pic_md5_sum': item['PicMd5Sum']})
        else:
            self.pic_list.append(
                {
                    'pic_md5_sum': message['SendPicsInfo']
                    ['PicList'].pop('item')['PicMd5Sum']
                }
            )


class PicSysphotoEvent(BasePicEvent):
    __type__ = 'pic_sysphoto_event'


class PicPhotoOrAlbumEvent(BasePicEvent):
    __type__ = 'pic_photo_or_album_event'


class PicWeixinEvent(BasePicEvent):
    __type__ = 'pic_weixin_event'


class LocationSelectEvent(SimpleEvent):
    __type__ = 'location_select_event'
    location_x = StringEntry('SendLocationInfo.Location_X')
    location_y = StringEntry('SendLocationInfo.Location_Y')
    scale = StringEntry('SendLocationInfo.Scale')
    label = StringEntry('SendLocationInfo.Label')
    poi_name = StringEntry('SendLocationInfo.Poiname')


class ClickEvent(SimpleEvent):
    __type__ = 'click_event'


class ViewEvent(SimpleEvent):
    __type__ = 'view_event'


class LocationEvent(WeChatEvent):
    __type__ = 'location_event'
    latitude = FloatEntry('Latitude')
    longitude = FloatEntry('Longitude')
    precision = FloatEntry('Precision')


class TemplateSendJobFinishEvent(WeChatEvent):
    __type__ = 'templatesendjobfinish_event'
    status = StringEntry('Status')


class BaseProductEvent(WeChatEvent):
    key_standard = StringEntry('KeyStandard')
    key_str = StringEntry('KeyStr')
    ext_info = StringEntry('ExtInfo')


class UserScanProductEvent(BaseProductEvent):
    __type__ = 'user_scan_product_event'
    country = StringEntry('Country')
    province = StringEntry('Province')
    city = StringEntry('City')
    sex = IntEntry('Sex')
    scene = IntEntry('Scene')


class UserScanProductEnterSessionEvent(BaseProductEvent):
    __type__ = 'user_scan_product_enter_session_event'


class UserScanProductAsyncEvent(BaseProductEvent):
    __type__ = 'user_scan_product_async_event'
    region_code = StringEntry('RegionCode')


class UserScanProductVerifyActionEvent(WeChatEvent):
    __type__ = 'user_scan_product_verify_action_event'
    key_standard = StringEntry('KeyStandard')
    key_str = StringEntry('KeyStr')
    result = StringEntry('Result')
    reason_msg = StringEntry('ReasonMsg')


class BaseCardCheckEvent(WeChatEvent):
    card_id = StringEntry('CardId')
    refuse_reason = StringEntry('RefuseReason')


class CardPassCheckEvent(BaseCardCheckEvent):
    __type__ = 'card_pass_check_event'


class CardNotPassCheckEvent(BaseCardCheckEvent):
    __type__ = 'card_not_pass_check_event'


class BaseCardEvent(WeChatEvent):
    card_id = StringEntry('CardId')
    user_card_code = StringEntry('UserCardCode')


class UserGetCardEvent(BaseCardEvent):
    __type__ = 'user_get_card_event'
    is_give_by_friend = IntEntry('IsGiveByFriend')
    friend_user_name = StringEntry('FriendUserName')
    outer_id = IntEntry('OuterId')
    old_user_card_code = StringEntry('OldUserCardCode')
    outer_str = StringEntry('OuterStr')
    is_restore_member_card = IntEntry('IsRestoreMemberCard')
    is_recommend_by_friend = IntEntry('IsRecommendByFriend')


class UserGiftingCardEvent(BaseCardEvent):
    __type__ = 'user_gifting_card_event'
    is_return_back = IntEntry('IsReturnBack')
    friend_user_name = StringEntry('FriendUserName')
    is_chat_room = IntEntry('IsChatRoom')


class UserDelCardEvent(BaseCardEvent):
    __type__ = 'user_del_card_event'


class UserConsumeCardEvent(BaseCardEvent):
    __type__ = 'user_consume_card_event'
    consume_source = StringEntry('ConsumeSource')
    location_name = StringEntry('LocationName')
    staff_open_id = StringEntry('StaffOpenId')
    verify_code = StringEntry('VerifyCode')
    remark_amount = StringEntry('RemarkAmount')
    outer_str = StringEntry('OuterStr')


class UserPayFromPayCellEvent(BaseCardEvent):
    __type__ = 'user_pay_from_pay_cell_event'
    trans_id = StringEntry('TransId')
    location_id = IntEntry('LocationId')
    fee = StringEntry('Fee')
    original_fee = StringEntry('OriginalFee')


class UserViewCardEvent(BaseCardEvent):
    __type__ = 'user_view_card_event'
    outer_str = StringEntry('OuterStr')


class UserEnterSessionFromCardEvent(BaseCardEvent):
    __type__ = 'user_enter_session_from_card_event'


class UpdateMemberCardEvent(BaseCardEvent):
    __type__ = 'update_member_card_event'
    modify_bonus = IntEntry('ModifyBonus')
    modify_balance = IntEntry('ModifyBalance')


class CardSkuRemindEvent(WeChatEvent):
    __type__ = 'card_sku_remind_event'
    card_id = StringEntry('CardId')
    detail = StringEntry('Detail')


class CardPayOrderEvent(WeChatEvent):
    __type__ = 'card_pay_order_event'
    order_id = StringEntry('OrderId')
    status = StringEntry('Status')
    create_order_time = IntEntry('CreateOrderTime')
    pay_finish_time = IntEntry('PayFinishTime')
    desc = StringEntry('Desc')
    free_coin_count = StringEntry('FreeCoinCount')
    pay_coin_count = StringEntry('PayCoinCount')
    refund_free_coin_count = StringEntry('RefundFreeCoinCount')
    refund_pay_coin_count = StringEntry('RefundPayCoinCount')
    order_type = StringEntry('OrderType')
    memo = StringEntry('Memo')
    receipt_info = StringEntry('ReceiptInfo')


class SubmitMembercardUserInfoEvent(BaseCardEvent):
    __type__ = 'submit_membercard_user_info_event'


class UnknownEvent(WeChatEvent):
    __type__ = 'unknown_event'



# -*- coding:utf-8 -*-

import hashlib
import time
import os
import pytest

from zgrobot import ZgRoBot
from zgrobot.utils import generate_token, to_text


def _make_xml(content):
    return """
        <xml>
        <ToUserName><![CDATA[toUser]]></ToUserName>
        <FromUserName><![CDATA[fromUser]]></FromUserName>
        <CreateTime>1348831860</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[%s]]></Content>
        <MsgId>1234567890123456</MsgId>
        </xml>
    """ % content


def test_signature_checker():
    token = generate_token()

    robot = ZgRoBot(token, SESSION_STORAGE=False)

    timestamp = str(int(time.time()))
    nonce = '12345678'

    sign = [token, timestamp, nonce]
    sign.sort()
    sign = ''.join(sign)
    sign = sign.encode()
    sign = hashlib.sha1(sign).hexdigest()

    assert robot.check_signature(timestamp, nonce, sign)


def test_register_handlers():  # noqa: C901
    robot = ZgRoBot(SESSION_STORAGE=False)

    for type in robot.message_types:
        assert hasattr(robot,
                       type) or hasattr(robot, type.replace('_event', ''))

    @robot.text
    def text_handler():
        return "Hi"

    assert robot._handlers["text"] == [(text_handler, 0)]

    @robot.image
    def image_handler(message):
        return 'nice pic'

    assert robot._handlers["image"] == [(image_handler, 1)]

    assert robot.get_handlers("text") == [(text_handler, 0)]

    @robot.handler
    def handler(message, session):
        pass

    assert robot.get_handlers("text") == [(text_handler, 0), (handler, 2)]

    @robot.video
    def video_handler():
        pass

    assert robot._handlers["video"] == [(video_handler, 0)]
    assert robot.get_handlers("video") == [(video_handler, 0), (handler, 2)]

    @robot.shortvideo
    def shortvideo_handler():
        pass

    assert robot._handlers["shortvideo"] == [(shortvideo_handler, 0)]
    assert robot.get_handlers("shortvideo") == [
        (shortvideo_handler, 0), (handler, 2)
    ]

    @robot.location
    def location_handler():
        pass

    assert robot._handlers["location"] == [(location_handler, 0)]

    @robot.link
    def link_handler():
        pass

    assert robot._handlers["link"] == [(link_handler, 0)]

    @robot.subscribe
    def subscribe_handler():
        pass

    assert robot._handlers["subscribe_event"] == [(subscribe_handler, 0)]

    @robot.unsubscribe
    def unsubscribe_handler():
        pass

    assert robot._handlers["unsubscribe_event"] == [(unsubscribe_handler, 0)]

    @robot.voice
    def voice_handler():
        pass

    assert robot._handlers["voice"] == [(voice_handler, 0)]

    @robot.click
    def click_handler():
        pass

    assert robot._handlers["click_event"] == [(click_handler, 0)]

    @robot.key_click("MENU")
    def menu_handler():
        pass

    assert len(robot._handlers["click_event"]) == 2

    @robot.scan
    def scan_handler():
        pass

    assert robot._handlers["scan_event"] == [(scan_handler, 0)]

    @robot.scancode_push
    def scancode_push_handler():
        pass

    assert robot._handlers["scancode_push_event"] == [
        (scancode_push_handler, 0)
    ]

    @robot.scancode_waitmsg
    def scancode_waitmsg_handler():
        pass

    assert robot._handlers["scancode_waitmsg_event"] == [
        (scancode_waitmsg_handler, 0)
    ]


def test_filter():
    import re
    import zgrobot.testing
    robot = ZgRoBot(SESSION_STORAGE=False)

    @robot.filter("喵")
    def _1():
        return "喵"

    assert len(robot._handlers["text"]) == 1

    @robot.filter(re.compile(to_text(".*?呵呵.*?")))
    def _2():
        return "哼"

    assert len(robot._handlers["text"]) == 2

    @robot.text
    def _3():
        return "汪"

    assert len(robot._handlers["text"]) == 3

    tester = zgrobot.testing.WeTest(robot)

    assert tester.send_xml(_make_xml("啊"))._args['content'] == u"汪"
    assert tester.send_xml(_make_xml("啊呵呵"))._args['content'] == u"哼"
    assert tester.send_xml(_make_xml("喵"))._args['content'] == u"喵"

    try:
        os.remove(os.path.abspath("zgrobot_session"))
    except OSError:
        pass
    robot = ZgRoBot(SESSION_STORAGE=False)

    @robot.filter("帮助", "跪求帮助", re.compile("(.*?)help.*?"))
    def _(message, session, match):
        if match and match.group(1) == u"小姐姐":
            return "本小姐就帮你一下"
        return "就不帮"

    assert len(robot._handlers["text"]) == 3

    @robot.text
    def _4():
        return "哦"

    assert len(robot._handlers["text"]) == 4

    tester = zgrobot.testing.WeTest(robot)

    assert tester.send_xml(_make_xml("啊"))._args['content'] == u"哦"
    assert tester.send_xml(_make_xml("帮助"))._args['content'] == u"就不帮"
    assert tester.send_xml(_make_xml("跪求帮助"))._args['content'] == u"就不帮"
    assert tester.send_xml(_make_xml("ooohelp"))._args['content'] == u"就不帮"
    assert tester.send_xml(_make_xml("小姐姐help")
                           )._args['content'] == u"本小姐就帮你一下"


def test_register_not_callable_object():
    robot = ZgRoBot(SESSION_STORAGE=False)
    with pytest.raises(ValueError):
        robot.add_handler("s")


def test_error_page():
    robot = ZgRoBot()

    @robot.error_page
    def make_error_page(url):
        return url

    assert robot.make_error_page('喵') == '喵'


def test_config_ignore():
    from zgrobot.config import Config
    config = Config(TOKEN="token from config")
    robot = ZgRoBot(config=config, token="token2333")
    assert robot.token == "token from config"


def test_add_filter():
    import zgrobot.testing
    import re

    robot = ZgRoBot()

    def test_register():
        return "test"

    robot.add_filter(test_register, ["test", re.compile(u".*?啦.*?")])

    tester = zgrobot.testing.WeTest(robot)

    assert tester.send_xml(_make_xml("test"))._args["content"] == "test"
    assert tester.send_xml(_make_xml(u"我要测试啦"))._args["content"] == "test"
    assert tester.send_xml(_make_xml(u"我要测试")) is None

    with pytest.raises(ValueError) as e:
        robot.add_filter("test", ["test"])
    assert e.value.args[0] == "test is not callable"

    with pytest.raises(ValueError) as e:
        robot.add_filter(test_register, "test")
    assert e.value.args[0] == "test is not list"

    with pytest.raises(TypeError) as e:
        robot.add_filter(test_register, [["bazinga"]])
    assert e.value.args[0] == "[\'bazinga\'] is not a valid rule"



# -*- coding: utf-8 -*-

from zgrobot.messages.entries import StringEntry, IntEntry, FloatEntry
from zgrobot.messages.base import ZgRoBotMetaClass


class MessageMetaClass(ZgRoBotMetaClass):
    pass


class WeChatMessage(object, metaclass=MessageMetaClass):
    message_id = IntEntry('MsgId', 0)
    target = StringEntry('ToUserName')
    source = StringEntry('FromUserName')
    time = IntEntry('CreateTime', 0)

    def __init__(self, message):
        self.__dict__.update(message)


class TextMessage(WeChatMessage):
    __type__ = 'text'
    content = StringEntry('Content')


class ImageMessage(WeChatMessage):
    __type__ = 'image'
    img = StringEntry('PicUrl')


class LocationMessage(WeChatMessage):
    __type__ = 'location'
    location_x = FloatEntry('Location_X')
    location_y = FloatEntry('Location_Y')
    label = StringEntry('Label')
    scale = IntEntry('Scale')

    @property
    def location(self):
        return self.location_x, self.location_y


class LinkMessage(WeChatMessage):
    __type__ = 'link'
    title = StringEntry('Title')
    description = StringEntry('Description')
    url = StringEntry('Url')


class VoiceMessage(WeChatMessage):
    __type__ = 'voice'
    media_id = StringEntry('MediaId')
    format = StringEntry('Format')
    recognition = StringEntry('Recognition')


class VideoMessage(WeChatMessage):
    __type__ = ['video', 'shortvideo']
    media_id = StringEntry('MediaId')
    thumb_media_id = StringEntry('ThumbMediaId')


class UnknownMessage(WeChatMessage):
    __type__ = 'unknown'



# -*- coding:utf-8 -*-

import time
import pytest

from zgrobot.parser import parse_user_msg
from zgrobot.replies import WeChatReply, TextReply, ImageReply, MusicReply
from zgrobot.replies import VoiceReply, VideoReply
from zgrobot.replies import Article, ArticlesReply
from zgrobot.replies import TransferCustomerServiceReply, SuccessReply
from zgrobot.replies import process_function_reply
from zgrobot.utils import to_binary, to_text


def test_wechat_reply():
    message = parse_user_msg(
        """
        <xml>
        <ToUserName><![CDATA[toUser]]></ToUserName>
        <FromUserName><![CDATA[fromUser]]></FromUserName>
        <CreateTime>1348831860</CreateTime>
        <MsgType><![CDATA[image]]></MsgType>
        <PicUrl><![CDATA[this is a url]]></PicUrl>
        <MediaId><![CDATA[media_id]]></MediaId>
        <MsgId>1234567890123456</MsgId>
        </xml>
    """
    )
    s = to_binary("喵fdsjaklfsk")
    reply = WeChatReply(message=message, s=s)
    assert reply._args['source'] == 'toUser'
    assert reply._args['target'] == 'fromUser'
    assert reply._args['s'] == to_text(s)
    assert isinstance(reply._args['time'], int)


def test_text_reply():
    t = int(time.time())
    reply = TextReply(target='fromUser', source='toUser', content="aa", time=t)
    assert reply.render().strip() == """
    <xml>
    <ToUserName><![CDATA[fromUser]]></ToUserName>
    <FromUserName><![CDATA[toUser]]></FromUserName>
    <CreateTime>{time}</CreateTime>
    <MsgType><![CDATA[text]]></MsgType>
    <Content><![CDATA[aa]]></Content>
    </xml>""".format(time=t).strip()


def test_image_reply():
    t = int(time.time())
    reply = ImageReply(
        target='fromUser', source='toUser', media_id="fdasfdasfasd", time=t
    )
    assert reply.render().strip() == """
    <xml>
    <ToUserName><![CDATA[fromUser]]></ToUserName>
    <FromUserName><![CDATA[toUser]]></FromUserName>
    <CreateTime>{time}</CreateTime>
    <MsgType><![CDATA[image]]></MsgType>
    <Image>
    <MediaId><![CDATA[fdasfdasfasd]]></MediaId>
    </Image>
    </xml>""".format(time=t).strip()


def test_voice_reply():
    t = int(time.time())
    reply = VoiceReply(
        target='tgu', source='su', media_id="fdasfdasfasd", time=t
    )
    assert reply.render().strip() == """
    <xml>
    <ToUserName><![CDATA[tgu]]></ToUserName>
    <FromUserName><![CDATA[su]]></FromUserName>
    <CreateTime>{time}</CreateTime>
    <MsgType><![CDATA[voice]]></MsgType>
    <Voice>
    <MediaId><![CDATA[fdasfdasfasd]]></MediaId>
    </Voice>
    </xml>""".format(time=t).strip()


def test_video_reply():
    t = int(time.time())
    reply = VideoReply(
        target='tgu',
        source='su',
        media_id="fdasfdasfasd",
        time=t,
    )
    assert reply.render().strip() == """
    <xml>
    <ToUserName><![CDATA[tgu]]></ToUserName>
    <FromUserName><![CDATA[su]]></FromUserName>
    <CreateTime>{time}</CreateTime>
    <MsgType><![CDATA[video]]></MsgType>
    <Video>
    <MediaId><![CDATA[fdasfdasfasd]]></MediaId>
    <Title><![CDATA[]]></Title>
    <Description><![CDATA[]]></Description>
    </Video>
    </xml>""".format(time=t).strip()

    reply_2 = VideoReply(
        target='tgu',
        source='su',
        media_id="fdasfdasfasd",
        time=t,
        title='meow'
    )

    assert reply_2.render().strip() == """
    <xml>
    <ToUserName><![CDATA[tgu]]></ToUserName>
    <FromUserName><![CDATA[su]]></FromUserName>
    <CreateTime>{time}</CreateTime>
    <MsgType><![CDATA[video]]></MsgType>
    <Video>
    <MediaId><![CDATA[fdasfdasfasd]]></MediaId>
    <Title><![CDATA[meow]]></Title>
    <Description><![CDATA[]]></Description>
    </Video>
    </xml>""".format(time=t).strip()

    reply_3 = VideoReply(
        target='tgu',
        source='su',
        media_id="fdasfdasfasd",
        time=t,
        title='meow',
        description='www'
    )
    assert reply_3.render().strip() == """
    <xml>
    <ToUserName><![CDATA[tgu]]></ToUserName>
    <FromUserName><![CDATA[su]]></FromUserName>
    <CreateTime>{time}</CreateTime>
    <MsgType><![CDATA[video]]></MsgType>
    <Video>
    <MediaId><![CDATA[fdasfdasfasd]]></MediaId>
    <Title><![CDATA[meow]]></Title>
    <Description><![CDATA[www]]></Description>
    </Video>
    </xml>""".format(time=t).strip()


def test_video_reply_process_args():
    reply = VideoReply(
        target='tgu',
        source='su',
        media_id="fdasfdasfasd",
    )
    assert reply._args['title'] == ''
    assert reply._args['description'] == ''


def test_music_reply():
    t = int(time.time())
    reply = MusicReply(
        target='tg',
        source='ss',
        time=t,
        title='tt',
        description='ds',
        url='u1',
        hq_url='u2',
    )
    assert reply.render().strip() == """
    <xml>
    <ToUserName><![CDATA[tg]]></ToUserName>
    <FromUserName><![CDATA[ss]]></FromUserName>
    <CreateTime>{time}</CreateTime>
    <MsgType><![CDATA[music]]></MsgType>
    <Music>
    <Title><![CDATA[tt]]></Title>
    <Description><![CDATA[ds]]></Description>
    <MusicUrl><![CDATA[u1]]></MusicUrl>
    <HQMusicUrl><![CDATA[u2]]></HQMusicUrl>
    </Music>
    </xml>""".format(time=t).strip()


def test_music_reply_process_args():
    reply = MusicReply(
        target='tg',
        source='ss',
        title='tt',
        description='ds',
        url='u1',
    )
    assert reply._args['hq_url'] == 'u1'

    reply_2 = MusicReply(
        target='tg',
        source='ss',
        title='tt',
        description='ds',
        url='u1',
        hq_url='u2'
    )
    assert reply_2._args['hq_url'] == 'u2'


def test_article():
    article = Article(
        title="tt", description=to_binary("附近的萨卡里发生"), img="http", url="uuu"
    )
    assert article.render().strip() == to_text(
        """
    <item>
    <Title><![CDATA[tt]]></Title>
    <Description><![CDATA[附近的萨卡里发生]]></Description>
    <PicUrl><![CDATA[http]]></PicUrl>
    <Url><![CDATA[uuu]]></Url>
    </item>
    """
    ).strip()


def test_articles_reply():
    article = Article(
        title="tt", description="附近的萨卡里发生", img="http", url="uuu"
    )
    t = int(time.time())
    reply = ArticlesReply(target='tg', source='ss', time=t)
    assert reply.render().strip() == """
    <xml>
    <ToUserName><![CDATA[tg]]></ToUserName>
    <FromUserName><![CDATA[ss]]></FromUserName>
    <CreateTime>{time}</CreateTime>
    <MsgType><![CDATA[news]]></MsgType>
    <Content><![CDATA[]]></Content>
    <ArticleCount>0</ArticleCount>
    <Articles></Articles>
    </xml>""".format(time=t).strip()

    reply._args['content'] = 'wwww'
    assert '<Content><![CDATA[wwww]]></Content>' in reply.render()
    reply.add_article(article)
    assert '<ArticleCount>1</ArticleCount>' in reply.render()
    assert article.render() in reply.render()
    for _ in range(9):
        reply.add_article(article)
    assert '<ArticleCount>10</ArticleCount>' in reply.render()
    with pytest.raises(AttributeError):
        reply.add_article(article)


def test_transfer_customer_service_reply():
    t = int(time.time())
    reply = TransferCustomerServiceReply(
        source='aaa', target='bbb', time=t, account="test1@test"
    )
    assert reply.render().strip() == """
            <xml>
            <ToUserName><![CDATA[bbb]]></ToUserName>
            <FromUserName><![CDATA[aaa]]></FromUserName>
            <CreateTime>{time}</CreateTime>
            <MsgType><![CDATA[transfer_customer_service]]></MsgType>
            <TransInfo>
                 <KfAccount><![CDATA[test1@test]]></KfAccount>
             </TransInfo>
            </xml>
    """.format(time=t).strip()


def test_transfer_customer_service_reply_without_account():
    t = int(time.time())
    reply = TransferCustomerServiceReply(source='aaa', target='bbb', time=t)
    assert reply.render().strip() == """
            <xml>
            <ToUserName><![CDATA[bbb]]></ToUserName>
            <FromUserName><![CDATA[aaa]]></FromUserName>
            <CreateTime>{time}</CreateTime>
            <MsgType><![CDATA[transfer_customer_service]]></MsgType>
            </xml>
        """.format(time=t).strip()


def test_success_reply():
    assert SuccessReply().render() == "success"


def test_process_unknown_function_reply():
    reply = SuccessReply()
    assert process_function_reply(reply) == reply


def test_process_text_function_reply():
    reply = process_function_reply("test")
    assert isinstance(reply, TextReply)
    assert reply.content == "test"


def test_process_music_function_reply():
    reply = process_function_reply(["title", "desc", "url"])
    assert isinstance(reply, MusicReply)
    assert reply.title == "title"
    assert reply.description == "desc"
    assert reply.url == reply.hq_url == "url"

    reply = process_function_reply(["title", "desc", "url", "hq"])
    assert isinstance(reply, MusicReply)
    assert reply.title == "title"
    assert reply.description == "desc"
    assert reply.url == "url"
    assert reply.hq_url == "hq"


def test_process_articles_function_reply():
    reply = process_function_reply(
        [["tt1", 'ds1', 'img', 'url'], ["tt2", 'ds2', 'im2g', 'u2rl']]
    )
    assert isinstance(reply, ArticlesReply)
    assert len(reply._articles) == 2
    article_1, article_2 = reply._articles
    assert isinstance(article_1, Article), isinstance(article_2, Article)
    assert article_1.title == "tt1", article_2.title == "tt2"
    assert article_1.description == "ds1", article_2.description == "ds2"
    assert article_1.img == "img", article_2.img == "im2g"
    assert article_1.url == "url", article_2.url == "u2rl"

    process_function_reply([[1, 2, 3, 4]] * 10)

    with pytest.raises(AttributeError):
        process_function_reply([[1, 2, 3, 4]] * 11)



# -*- coding: utf-8 -*-

import os
import random
import sys
import time

import pytest
import tornado
import webtest
from tornado.testing import AsyncHTTPSTestCase
from webtest.app import AppError

from zgrobot.parser import parse_xml, process_message
from zgrobot.utils import generate_token, get_signature


@pytest.fixture
def wsgi_tester():
    def tester(app, token, endpoint):
        test_app = webtest.TestApp(app)

        response = test_app.get(endpoint, expect_errors=True)
        assert response.status_code == 403

        timestamp = str(time.time())
        nonce = str(random.randint(0, 10000))
        signature = get_signature(token, timestamp, nonce)
        echostr = generate_token()

        params = "?timestamp=%s&nonce=%s&signature=%s&echostr=%s" % (
            timestamp, nonce, signature, echostr
        )
        response = test_app.get(endpoint + params)

        assert response.status_code == 200
        assert response.body.decode('utf-8') == echostr

        response = test_app.get(endpoint, expect_errors=True)

        assert response.status_code == 403
        assert response.body.decode('utf-8') == u'喵'

        xml = """
                <xml>
                    <ToUserName><![CDATA[toUser]]></ToUserName>
                    <FromUserName><![CDATA[fromUser]]></FromUserName>
                    <CreateTime>1348831860</CreateTime>
                    <MsgType><![CDATA[text]]></MsgType>
                    <Content><![CDATA[this is a test]]></Content>
                    <MsgId>1234567890123456</MsgId>
                </xml>
                """
        with pytest.raises(AppError):
            # WebTest will raise an AppError
            # if the status_code is not >= 200 and < 400.
            test_app.post(endpoint, xml, content_type="text/xml")

        response = test_app.post(
            endpoint + params, xml, content_type="text/xml"
        )

        assert response.status_code == 200
        response = process_message(parse_xml(response.body))
        assert response.content == 'hello'

    return tester


@pytest.fixture(scope="module")
def hello_robot():
    from zgrobot import ZgRoBot
    robot = ZgRoBot(token='', SESSION_STORAGE=False)

    @robot.text
    def hello():
        return 'hello'

    @robot.error_page
    def make_error_page(url):
        return '喵'

    return robot


def test_django():
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "django_test.settings")
    sys.path.append(
        os.path.join(
            os.path.abspath(os.path.dirname(__file__)), 'django_test_env'
        )
    )

    from django.test.utils import setup_test_environment
    setup_test_environment()
    from django.test.client import Client
    from zgrobot.parser import parse_xml, process_message
    import django

    django.setup()
    client = Client()

    token = 'TestDjango'
    timestamp = str(time.time())
    nonce = str(random.randint(0, 10000))
    signature = get_signature(token, timestamp, nonce)
    echostr = generate_token()

    response = client.get(
        '/robot/', {
            'signature': signature,
            'timestamp': timestamp,
            'nonce': nonce,
            'echostr': echostr
        }
    )
    assert response.status_code == 200
    assert response.content.decode('utf-8') == echostr

    xml = """
    <xml>
        <ToUserName><![CDATA[toUser]]></ToUserName>
        <FromUserName><![CDATA[fromUser]]></FromUserName>
        <CreateTime>1348831860</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[this is a test]]></Content>
        <MsgId>1234567890123456</MsgId>
    </xml>"""
    params = "?timestamp=%s&nonce=%s&signature=%s" % \
             (timestamp, nonce, signature)
    url = '/robot/'
    response = client.post(url, data=xml, content_type="text/xml")

    assert response.status_code == 403
    assert response.content.decode('utf-8') == u'喵'

    url += params
    response = client.post(url, data=xml, content_type="text/xml")

    assert response.status_code == 200
    response = process_message(parse_xml(response.content))
    assert response.content == 'hello'

    response = client.options(url)
    assert response.status_code == 405


def test_flask(wsgi_tester, hello_robot):
    from flask import Flask
    from zgrobot.contrib.flask import make_view

    token = generate_token()
    endpoint = '/zgrobot_flask'

    hello_robot.token = token
    flask_app = Flask(__name__)
    flask_app.debug = True

    flask_app.add_url_rule(
        rule=endpoint,
        endpoint='zgrobot',
        view_func=make_view(hello_robot),
        methods=['GET', 'POST']
    )

    wsgi_tester(flask_app, token=token, endpoint=endpoint)


def test_bottle(wsgi_tester, hello_robot):
    from zgrobot.contrib.bottle import make_view
    from bottle import Bottle

    token = generate_token()
    endpoint = '/zgrobot_bottle'

    hello_robot.token = token

    bottle_app = Bottle()
    bottle_app.route(endpoint, ['GET', 'POST'], make_view(hello_robot))

    wsgi_tester(bottle_app, token=token, endpoint=endpoint)


def test_zgrobot_wsgi(wsgi_tester, hello_robot):
    token = generate_token()
    endpoint = r'/rand'
    hello_robot.token = token

    wsgi_tester(hello_robot.wsgi, token=token, endpoint=endpoint)


# workaround to make Tornado work in Python 3.8
# https://github.com/tornadoweb/tornado/issues/2608
if sys.platform == 'win32' and sys.version_info >= (3, 8):
    import asyncio
    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())

if tornado.version_info[0] < 6:

    def test_tornado(wsgi_tester, hello_robot):
        from tornado.wsgi import WSGIAdapter
        import tornado.web
        from zgrobot.contrib.tornado import make_handler

        token = generate_token()
        endpoint = r'/zgrobot_tornado'
        hello_robot.token = token

        tornado_app = tornado.web.Application(
            [
                (endpoint, make_handler(hello_robot)),
            ], debug=True
        )
        wsgi_tester(WSGIAdapter(tornado_app), token=token, endpoint=endpoint)
else:

    class TestTornado(AsyncHTTPSTestCase):
        token = 'TestTornado'
        endpoint = '/zgrobot_tornado'

        @property
        def robot(self):
            from zgrobot import ZgRoBot
            robot = ZgRoBot(token=self.token, SESSION_STORAGE=False)

            @robot.text
            def hello():
                return 'hello'

            @robot.error_page
            def make_error_page(url):
                return '喵'

            return robot

        def get_app(self):
            import tornado.web
            from zgrobot.contrib.tornado import make_handler

            tornado_app = tornado.web.Application(
                [
                    (self.endpoint, make_handler(self.robot)),
                ], debug=True
            )
            return tornado_app

        def test_tornado(self):
            token = self.token
            timestamp = str(time.time())
            nonce = str(random.randint(0, 10000))
            signature = get_signature(token, timestamp, nonce)
            echostr = generate_token()

            params = "?timestamp=%s&nonce=%s&signature=%s&echostr=%s" % (
                timestamp, nonce, signature, echostr
            )

            response = self.fetch(path=self.endpoint + params)
            assert response.code == 200
            assert response.body.decode('utf-8') == echostr

            response = self.fetch(path=self.endpoint, )
            assert response.code == 403
            assert response.body.decode('utf-8') == u'喵'

            xml = """
            <xml>
                <ToUserName><![CDATA[toUser]]></ToUserName>
                <FromUserName><![CDATA[fromUser]]></FromUserName>
                <CreateTime>1348831860</CreateTime>
                <MsgType><![CDATA[text]]></MsgType>
                <Content><![CDATA[this is a test]]></Content>
                <MsgId>1234567890123456</MsgId>
            </xml>"""

            response = self.fetch(
                path=self.endpoint + params,
                method='POST',
                body=xml,
                headers={'Content-Type': 'text/xml'}
            )
            self.assertEqual(response.code, 200)
            self.assertEqual(
                process_message(parse_xml(response.body)).content, 'hello'
            )

            response = self.fetch(
                path=self.endpoint,
                method='POST',
                body=xml,
                headers={'Content-Type': 'text/xml'}
            )
            self.assertEqual(response.code, 403)



# -*- coding: utf-8 -*-

from zgrobot.session import SessionStorage
from zgrobot.utils import json_loads, json_dumps

__CREATE_TABLE_SQL__ = """
CREATE TABLE IF NOT EXISTS ZgRoBot(
id VARCHAR(100) NOT NULL ,
value BLOB NOT NULL,
PRIMARY KEY (id)
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
"""


class MySQLStorage(SessionStorage):
    """
    MySQLStorage 会把你的 Session 数据储存在 MySQL 中 ::

        import MySQLdb # 使用 mysqlclient
        import zgrobot
        from zgrobot.session.mysqlstorage import MySQLStorage

        conn = MySQLdb.connect(user='', db='', passwd='', host='')
        session_storage = MySQLStorage(conn)
        robot = zgrobot.ZgRoBot(token="token", enable_session=True,
                                session_storage=session_storage)

    或者 ::

        import pymysql # 使用 pymysql
        import zgrobot
        from zgrobot.session.mysqlstorage import MySQLStorage

        session_storage = MySQLStorage(
        conn=pymysql.connect(
            user='喵',
            password='喵喵',
            db='zgrobot',
            host='127.0.0.1',
            charset='utf8'
        ))
        robot = zgrobot.ZgRoBot(token="token", enable_session=True,
                                session_storage=session_storage)

    你需要安装一个 MySQL Client 才能使用 MySQLStorage，比如 ``pymysql``，``mysqlclient`` 。

    理论上符合 `PEP-249 <https://www.python.org/dev/peps/pep-0249/#connection-objects>`_ 的库都可以使用，\
    测试时使用的是 ``pymysql``。

    :param conn: `PEP-249 <https://www.python.org/dev/peps/pep-0249/#connection-objects>`_\
    定义的 Connection 对象
    """
    def __init__(self, conn):
        self.conn = conn
        self.conn.cursor().execute(__CREATE_TABLE_SQL__)

    def get(self, id):
        """
        根据 id 获取数据。

        :param id: 要获取的数据的 id
        :return: 返回取到的数据，如果是空则返回一个空的 ``dict`` 对象
        """
        cur = self.conn.cursor()
        cur.execute("SELECT value FROM ZgRoBot WHERE id=%s LIMIT 1;", (id, ))
        session_json = cur.fetchone()
        if session_json is None:
            return {}
        return json_loads(session_json[0])

    def set(self, id, value):
        """
        根据 id 写入数据。

        :param id: 要写入的 id
        :param value: 要写入的数据，可以是一个 ``dict`` 对象
        """
        value = json_dumps(value)
        self.conn.cursor().execute(
            "INSERT INTO ZgRoBot (id, value) VALUES (%s,%s) \
                ON DUPLICATE KEY UPDATE value=%s", (
                id,
                value,
                value,
            )
        )
        self.conn.commit()

    def delete(self, id):
        """
        根据 id 删除数据。

        :param id: 要删除的数据的 id
        """
        self.conn.cursor().execute("DELETE FROM ZgRoBot WHERE id=%s", (id, ))
        self.conn.commit()



# -*- coding: utf-8 -*-

import base64
import socket
import struct
import time

try:
    from cryptography.hazmat.primitives.ciphers import (
        Cipher, algorithms, modes
    )
    from cryptography.hazmat.backends import default_backend
except ImportError:  # pragma: no cover
    raise RuntimeError("You need to install Cryptography.")  # pragma: no cover

from . import pkcs7
from .exceptions import (
    UnvalidEncodingAESKey, AppIdValidationError, InvalidSignature
)
from zgrobot.utils import (
    to_text, to_binary, generate_token, byte2int, get_signature
)


class PrpCrypto(object):
    """
    提供接收和推送给公众平台消息的加解密接口
    """
    def __init__(self, key):
        key = to_binary(key)
        self.cipher = Cipher(
            algorithms.AES(key),
            modes.CBC(key[:16]),
            backend=default_backend()
        )

    def get_random_string(self):
        """
        :return: 长度为16的随即字符串
        """
        return generate_token(16)

    def encrypt(self, text, app_id):
        """
        对明文进行加密
        :param text: 需要加密的明文
        :param app_id: 微信公众平台的 AppID
        :return: 加密后的字符串
        """
        text = b"".join(
            [
                to_binary(self.get_random_string()),
                struct.pack(b"I", socket.htonl(len(to_binary(text)))),
                to_binary(text),
                to_binary(app_id)
            ]
        )
        text = pkcs7.encode(text)
        encryptor = self.cipher.encryptor()
        ciphertext = to_binary(encryptor.update(text) + encryptor.finalize())
        return base64.b64encode(ciphertext)

    def decrypt(self, text, app_id):
        """
        对密文进行解密
        :param text: 需要解密的密文
        :param app_id: 微信公众平台的 AppID
        :return: 解密后的字符串
        """
        text = to_binary(text)
        decryptor = self.cipher.decryptor()
        plain_text = decryptor.update(base64.b64decode(text)
                                      ) + decryptor.finalize()

        padding = byte2int(plain_text, -1)
        content = plain_text[16:-padding]

        xml_len = socket.ntohl(struct.unpack("I", content[:4])[0])
        xml_content = content[4:xml_len + 4]
        from_appid = content[xml_len + 4:]

        if to_text(from_appid) != app_id:
            raise AppIdValidationError(text, app_id)

        return xml_content


class MessageCrypt(object):
    ENCRYPTED_MESSAGE_XML = """
<xml>
<Encrypt><![CDATA[{encrypt}]]></Encrypt>
<MsgSignature><![CDATA[{signature}]]></MsgSignature>
<TimeStamp>{timestamp}</TimeStamp>
<Nonce><![CDATA[{nonce}]]></Nonce>
</xml>
    """.strip()

    def __init__(self, token, encoding_aes_key, app_id):
        key = base64.b64decode(to_binary(encoding_aes_key + '='))
        if len(key) != 32:
            raise UnvalidEncodingAESKey(encoding_aes_key)
        self.prp_crypto = PrpCrypto(key)

        self.token = token
        self.app_id = app_id

    def decrypt_message(self, timestamp, nonce, msg_signature, encrypt_msg):
        """
        解密收到的微信消息
        :param timestamp: 请求 URL 中收到的 timestamp
        :param nonce: 请求 URL 中收到的 nonce
        :param msg_signature: 请求 URL 中收到的 msg_signature
        :param encrypt_msg: 收到的加密文本. ( XML 中的 <Encrypt> 部分 )
        :return: 解密后的 XML 文本
        """
        signature = get_signature(self.token, timestamp, nonce, encrypt_msg)
        if signature != msg_signature:
            raise InvalidSignature(msg_signature)
        return self.prp_crypto.decrypt(encrypt_msg, self.app_id)

    def encrypt_message(self, reply, timestamp=None, nonce=None):
        """
        加密微信回复
        :param reply: 加密前的回复
        :type reply: WeChatReply 或 XML 文本
        :return: 加密后的回复文本
        """
        if hasattr(reply, "render"):
            reply = reply.render()

        timestamp = timestamp or to_text(int(time.time()))
        nonce = nonce or generate_token(5)
        encrypt = to_text(self.prp_crypto.encrypt(reply, self.app_id))
        signature = get_signature(self.token, timestamp, nonce, encrypt)
        return to_text(
            self.ENCRYPTED_MESSAGE_XML.format(
                encrypt=encrypt,
                signature=signature,
                timestamp=timestamp,
                nonce=nonce
            )
        )



__version__ = '2.0.0'
__author__ = 'pylover'
__license__ = 'MIT'

__all__ = ["ZgRoBot"]

try:
    from zgrobot.robot import ZgRoBot
except ImportError:  # pragma: no cover
    pass  # pragma: no cover



# -*- coding: utf-8 -*-
from bottle import request, HTTPResponse

import html


def make_view(robot):
    """
    为一个 BaseRoBot 生成 Bottle view。

    Usage ::

        from zgrobot import ZgRoBot

        robot = ZgRoBot(token='token')


        @robot.handler
        def hello(message):
            return 'Hello World!'

        from bottle import Bottle
        from zgrobot.contrib.bottle import make_view

        app = Bottle()
        app.route(
            '/robot',  # ZgRoBot 挂载地址
            ['GET', 'POST'],
            make_view(robot)
        )


    :param robot: 一个 BaseRoBot 实例
    :return: 一个标准的 Bottle view
    """
    def zgrobot_view(*args, **kwargs):
        if not robot.check_signature(
            request.query.timestamp, request.query.nonce,
            request.query.signature
        ):
            return HTTPResponse(
                status=403,
                body=robot.make_error_page(html.escape(request.url))
            )
        if request.method == 'GET':
            return request.query.echostr
        else:
            body = request.body.read()
            message = robot.parse_message(
                body,
                timestamp=request.query.timestamp,
                nonce=request.query.nonce,
                msg_signature=request.query.msg_signature
            )
            return robot.get_encrypted_reply(message)

    return zgrobot_view



# -*- coding: utf-8 -*-

from zgrobot.crypto import PrpCrypto, MessageCrypt
from zgrobot.utils import generate_token, to_binary, to_text
from zgrobot.parser import parse_xml
import time


def test_prpcrypto():
    key = "ReUrr0NKeHkppBQq"

    assert len(key) == 16

    crypto = PrpCrypto(key)
    text = generate_token(32)
    app_id = generate_token(32)
    assert crypto.decrypt(crypto.encrypt(text, app_id),
                          app_id) == to_binary(text)


def test_message_crypt():
    encoding_aes_key = generate_token(32) + generate_token(11)
    token = generate_token()
    timestamp = to_text(int(time.time()))
    nonce = generate_token(5)
    app_id = generate_token(18)
    crypt = MessageCrypt(
        token=token, encoding_aes_key=encoding_aes_key, app_id=app_id
    )

    message = crypt.encrypt_message('hello', timestamp, nonce)
    assert message is not None
    message = parse_xml(message)
    assert message is not None
    message = crypt.decrypt_message(
        message['TimeStamp'], message['Nonce'], message['MsgSignature'],
        message['Encrypt']
    )
    assert message == to_binary('hello')



import time
import logging

from zgrobot.logger import enable_pretty_logging, _LogFormatter


def get_new_logger():
    return logging.getLogger(str(time.time()))


def test_logger_level():
    for level in ('debug', 'info', 'warning', 'error'):
        logger = get_new_logger()
        enable_pretty_logging(logger, level=level)
        assert logger.level == getattr(logging, level.upper())


def test_handlers():
    logger = get_new_logger()
    enable_pretty_logging(logger)
    assert isinstance(logger.handlers[0].formatter, _LogFormatter)



# -*- coding: utf-8 -*-

from . import SessionStorage


class SaeKVDBStorage(SessionStorage):
    """
    SaeKVDBStorage 使用SAE 的 KVDB 来保存你的session ::

        import zgrobot
        from zgrobot.session.saekvstorage import SaeKVDBStorage

        session_storage = SaeKVDBStorage()
        robot = zgrobot.ZgRoBot(token="token", enable_session=True,
                                session_storage=session_storage)

    需要先在后台开启 KVDB 支持

    :param prefix: KVDB 中 Session 数据 key 的 prefix 。默认为 ``ws_``
    """
    def __init__(self, prefix='ws_'):
        try:
            import sae.kvdb
        except ImportError:
            raise RuntimeError("SaeKVDBStorage requires SAE environment")
        self.kv = sae.kvdb.KVClient()  # pragma: no cover
        self.prefix = prefix  # pragma: no cover

    def key_name(self, s):
        return '{prefix}{s}'.format(prefix=self.prefix, s=s)

    def get(self, id):
        """
        根据 id 获取数据。

        :param id: 要获取的数据的 id
        :return: 返回取到的数据，如果是空则返回一个空的 ``dict`` 对象
        """
        return self.kv.get(self.key_name(id)) or {}

    def set(self, id, value):
        """
        根据 id 写入数据。

        :param id: 要写入的 id
        :param value: 要写入的数据，可以是一个 ``dict`` 对象
        """
        return self.kv.set(self.key_name(id), value)

    def delete(self, id):
        """
        根据 id 删除数据。

        :param id: 要删除的数据的 id
        """
        return self.kv.delete(self.key_name(id))



class ZgRoBotMetaClass(type):
    TYPES = {}

    def __new__(mcs, name, bases, attrs):
        return type.__new__(mcs, name, bases, attrs)

    def __init__(cls, name, bases, attrs):
        if '__type__' in attrs:
            if isinstance(attrs['__type__'], list):
                for _type in attrs['__type__']:
                    cls.TYPES[_type] = cls
            else:
                cls.TYPES[attrs['__type__']] = cls
        type.__init__(cls, name, bases, attrs)



# -*- coding: utf-8 -*-

from zgrobot.session import SessionStorage
from zgrobot.utils import json_loads, json_dumps


class RedisStorage(SessionStorage):
    """
    RedisStorage 会把你的 Session 数据储存在 Redis 中 ::

        import redis
        import zgrobot
        from zgrobot.session.redisstorage import RedisStorage

        db = redis.Redis()
        session_storage = RedisStorage(db, prefix="my_prefix_")
        robot = zgrobot.ZgRoBot(token="token", enable_session=True,
                                session_storage=session_storage)


    你需要安装 ``redis`` 才能使用 RedisStorage 。

    :param redis: 一个 Redis Client。
    :param prefix: Reids 中 Session 数据 key 的 prefix 。默认为 ``ws_``
    """
    def __init__(self, redis, prefix='ws_'):
        for method_name in ['get', 'set', 'delete']:
            assert hasattr(redis, method_name)
        self.redis = redis
        self.prefix = prefix

    def key_name(self, s):
        return '{prefix}{s}'.format(prefix=self.prefix, s=s)

    def get(self, id):
        """
        根据 id 获取数据。

        :param id: 要获取的数据的 id
        :return: 返回取到的数据，如果是空则返回一个空的 ``dict`` 对象
        """
        id = self.key_name(id)
        session_json = self.redis.get(id) or '{}'
        return json_loads(session_json)

    def set(self, id, value):
        """
        根据 id 写入数据。

        :param id: 要写入的 id
        :param value: 要写入的数据，可以是一个 ``dict`` 对象
        """
        id = self.key_name(id)
        self.redis.set(id, json_dumps(value))

    def delete(self, id):
        """
        根据 id 删除数据。

        :param id: 要删除的数据的 id
        """
        id = self.key_name(id)
        self.redis.delete(id)



# -*- coding: utf-8 -*-

import xmltodict
from zgrobot.messages.messages import MessageMetaClass, UnknownMessage
from zgrobot.messages.events import EventMetaClass, UnknownEvent


def parse_user_msg(xml):
    message = process_message(parse_xml(xml)) if xml else None
    return message


def parse_xml(text):
    xml_dict = xmltodict.parse(text)["xml"]
    xml_dict["raw"] = text
    return xml_dict


def process_message(message):
    """
    Process a message dict and return a Message Object
    :param message: Message dict returned by `parse_xml` function
    :return: Message Object
    """
    message["type"] = message.pop("MsgType").lower()
    if message["type"] == 'event':
        message["type"] = str(message.pop("Event")).lower() + '_event'
        message_type = EventMetaClass.TYPES.get(message["type"], UnknownEvent)
    else:
        message_type = MessageMetaClass.TYPES.get(
            message["type"], UnknownMessage
        )
    return message_type(message)



# -*- coding: utf-8 -*-

from zgrobot.session import SessionStorage
from zgrobot.utils import json_loads, json_dumps


class MongoDBStorage(SessionStorage):
    """
    MongoDBStorage 会把你的 Session 数据储存在一个 MongoDB Collection 中 ::

        import pymongo
        import zgrobot
        from zgrobot.session.mongodbstorage import MongoDBStorage

        collection = pymongo.MongoClient()["wechat"]["session"]
        session_storage = MongoDBStorage(collection)
        robot = zgrobot.ZgRoBot(token="token", enable_session=True,
                                session_storage=session_storage)


    你需要安装 ``pymongo`` 才能使用 MongoDBStorage 。

    :param collection: 一个 MongoDB Collection。
    """
    def __init__(self, collection):
        self.collection = collection
        collection.create_index("wechat_id")

    def _get_document(self, id):
        return self.collection.find_one({"wechat_id": id})

    def get(self, id):
        """
        根据 id 获取数据。

        :param id: 要获取的数据的 id
        :return: 返回取到的数据，如果是空则返回一个空的 ``dict`` 对象
        """
        document = self._get_document(id)
        if document:
            session_json = document["session"]
            return json_loads(session_json)
        return {}

    def set(self, id, value):
        """
        根据 id 写入数据。

        :param id: 要写入的 id
        :param value: 要写入的数据，可以是一个 ``dict`` 对象
                """
        session = json_dumps(value)
        self.collection.replace_one(
            {"wechat_id": id}, {
                "wechat_id": id,
                "session": session
            },
            upsert=True
        )

    def delete(self, id):
        """
        根据 id 删除数据。

        :param id: 要删除的数据的 id
        """
        self.collection.delete_one({"wechat_id": id})
